/*
 * Copyright (c) 2014 Hugh Bailey <obs.jim@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#pragma once

#include "dstr.h"

class DStr {
	dstr str;

	DStr(DStr const &) = delete;
	DStr &operator=(DStr const &) = delete;

public:
	inline DStr() { dstr_init(&str); }
	inline DStr(DStr &&other) : DStr() { dstr_move(&str, &other.str); }

	inline DStr &operator=(DStr &&other)
	{
		dstr_move(&str, &other.str);
		return *this;
	}

	inline ~DStr() { dstr_free(&str); }

	inline operator dstr *() { return &str; }
	inline operator const dstr *() const { return &str; }

	inline operator char *() { return str.array; }
	inline operator const char *() const { return str.array; }

	inline dstr *operator->() { return &str; }
};
